Add Timepoints to a Timer
add_timepoints.RdAdds multiple enrollment and dropout events from a data frame.
Arguments
- timer
Timerinstance.- df
data.framewith columns:time(numeric),arm(character),enroller(integer),dropper(integer).
add_timepoints.RdAdds multiple enrollment and dropout events from a data frame.
Timer instance.
data.frame with columns: time (numeric), arm (character),
enroller (integer), dropper (integer).